#ifndef AUDIO_HPP_INCLUDED__
#define AUDIO_HPP_INCLUDED__


#include<SDL.h>
#include<cstdint>
#include<vector>
#include<list>
#include<random>




namespace audio{




enum class
WaveForm
{
  sine,
  square,
  triangle,
  sawtooth,

  END,

};


class
Wave
{
  const double  frequency_;

  std::vector<int16_t>  buffer_;

  void  generate_sine(    double  angular_frequency, int  length);
  void  generate_square(  double  angular_frequency, int  length);
  void  generate_triangle(                           int  length);
  void  generate_sawtooth(                           int  length);

public:
  Wave(WaveForm  form, double  device_frequency, double  frequency);

  const int16_t&  get_sample(int  pos) const;

  size_t  get_length() const;

   static int  get_value_max();
   static int  get_value_min();

};


class
Player
{
public:
  enum class Status{
    stopping,
    playing,
    pausing,
  };

private:
  const Wave&  wave_;

  int  position_;

  Status  status_;

  int  count_;

public:
  Player(const Wave&  wave, int  count);


  bool  mix(int16_t*  dst, int  len);

  void    set_status(Status  status)      ;
  Status  get_status(              ) const;

};


namespace device{


using WaveTable = std::vector<Wave>;


const WaveTable&  get_wave_table(WaveForm  form);

void   open(int  frequency);
void  close(              );

Player*  new_player(WaveForm  form, int  scale, int  ms);
void     new_noise(int  ms);

void  set_pause(bool  v);


}



}



#endif




