#include"audio.hpp"


#define rep(v,n)  for(int  v = 0;  v < (int)(n);  v += 1)


namespace audio{
namespace device{


namespace{
std::random_device::result_type
get_random_result()
{
  std::random_device  randev;

  return randev();
}


SDL_AudioSpec  obtained_;

SDL_AudioDeviceID  id_(0);

WaveTable  wave_table_[(int)WaveForm::END];

int  frequency_;

std::list<Player>  players_;
std::list<int>      noises_;

std::default_random_engine  random_engine_(get_random_result());

std::uniform_int_distribution<int16_t>  dist_(Wave::get_value_min(),Wave::get_value_max());




void
process_players(int16_t*  buf, int  len)
{
  auto   it = players_.begin();
  auto  end = players_.end();

    while(it != end)
    {
      auto&  player = *it;

        if(!player.mix(buf,len))
        {
          it = players_.erase(it);
        }

      else
        {
          ++it;
        }
    }


}


void
process_noises(int16_t*  buf, int  len)
{
  auto   it = noises_.begin();
  auto  end = noises_.end();

    while(it != end)
    {
      auto&  l = *it;

      int  minlen = std::min(l,len);

      l -= minlen;

      auto  p = buf;

        while(minlen--)
        {
          *p++ += dist_(random_engine_);
        }


        if(!l)
        {
          it = noises_.erase(it);
        }

      else
        {
          ++it;
        }
    }
}


void
callback(void*  userdata, uint8_t*  buf, int  len)
{
  SDL_memset(buf,obtained_.silence,len);

  auto  dst = (int16_t*)buf;

  len /= sizeof(int16_t);

  process_players(dst,len);
  process_noises( dst,len);
}




}




void
open(int  frequency)
{
    if(!id_)
    {
      SDL_InitSubSystem(SDL_INIT_AUDIO);

        rep(i,WaveForm::END)
        {
          double  f = 220.0;

            while(f <= 1761)
            {
              wave_table_[i].emplace_back((WaveForm)i,frequency,f);

              f *= 1.05946309436;
            }
        }


      SDL_AudioSpec  desired;

      frequency_ = frequency;

      desired.freq     =   frequency_;
      desired.samples  =         4096;
      desired.channels =            1;
      desired.format   = AUDIO_S16SYS;
      desired.callback = callback;

      id_ = SDL_OpenAudioDevice(nullptr,0,&desired,&obtained_,0);
    }
}


void
close()
{
    if(id_)
    {
      SDL_CloseAudioDevice(id_)   ;
                           id_ = 0;

      SDL_QuitSubSystem(SDL_INIT_AUDIO);
    }
}


const WaveTable&
get_wave_table(WaveForm  form)
{
  return wave_table_[(int)form];
}


Player*
new_player(WaveForm  form, int  scale, int  ms)
{
  players_.emplace_back(wave_table_[(int)form][scale],frequency_/1000*ms);

  return &players_.back();
}


void
new_noise(int  ms)
{
  noises_.emplace_back(frequency_/1000*ms);
}


void
set_pause(bool  v)
{
  SDL_PauseAudioDevice(id_,v);
}




}}





