#include"audio.hpp"
#include<cmath>




#define rep(v,n)  for(int  v = 0;  v < (int)(n);  v += 1)


namespace audio{




namespace{


const int  value_max = INT16_MAX/8;
const int  value_min = INT16_MIN/8;


using uint = unsigned int;


}


constexpr double
pi()
{
  return 3.14159265358979323846;
}


void
Wave::
generate_sine(double  angular_frequency, int  length)
{
    rep(i,length)
    {
      auto  v = static_cast<int16_t>(value_max*std::sin(angular_frequency*i));

      buffer_[i] = v;
    }
}


void
Wave::
generate_square(double  angular_frequency, int  length)
{
  generate_sine(angular_frequency,length);

    for(auto&  v: buffer_)
    {
      v = ((v >= 0)? value_max:value_min);
    }
}


void
Wave::
generate_triangle(int  length)
{
  const int  length_half    = (length/2);
  const int  length_quarter = (length/4);

        uint  v = 0;
  const uint  a = (value_max/length_quarter);

    rep(i,length_quarter)
    {
      buffer_[              i] = v;
      buffer_[length_half-1-i] = v;

      buffer_[length_half  +i] = -v;
      buffer_[length     -1-i] = -v;

      v += a;
    }
}


void
Wave::
generate_sawtooth(int  length)
{
  const int  length_half = (length/2);

        uint  v = 0;
  const uint  a = (value_max/length_half);

    rep(i,length_half)
    {
      buffer_[            i] =           v;
      buffer_[length_half+i] = value_min+v;

      v += a;
    }
}




Wave::
Wave(WaveForm  form, double  device_frequency, double  frequency):
frequency_(frequency)
{
  const auto  a_freq = (2*pi()/device_frequency)*frequency;

  const int  length = (device_frequency/frequency);

  buffer_.resize(length);

    switch(form)
    {
      case(WaveForm::sine    ): generate_sine(    a_freq,length);break;
      case(WaveForm::square  ): generate_square(  a_freq,length);break;
      case(WaveForm::triangle): generate_triangle(       length);break;
      case(WaveForm::sawtooth): generate_sawtooth(       length);break;
      default:;
    }
}




const int16_t&
Wave::
get_sample(int  pos) const
{
  return buffer_[pos];
}


size_t
Wave::
get_length() const
{
  return buffer_.size();
}


int  Wave::get_value_max(){return value_max;}
int  Wave::get_value_min(){return value_min;}


}




