#include"cube.hpp"




namespace cube{


namespace{


constexpr uint32_t
pixel = 0xFFFFFFFF;


void
draw_dline(Drawer&  dr, int  x, int  y, int  xmod, int  ymod)
{
  dr.draw_point(pixel,x,y);

  x += xmod;
  y += ymod;

    for(int  n = (side_length/2)-1;  n--;)
    {
      dr.draw_hline(pixel,x,y,2);

      x += xmod*2;
      y += ymod  ;

    }


  dr.draw_point(pixel,x,y);
}


void
draw_dline2(Drawer&  dr, int  x, int  y, int  xmod, int  ymod)
{
    for(int  n = (side_length/2);  n--;)
    {
      dr.draw_hline(pixel,x,y,2);

      x += xmod*2;
      y += ymod  ;
    }
}


}


void
draw_frame(Drawer&  dr, int  x, int  y, FrameID  fid)
{
    switch(fid)
    {
      case(FrameID::top_front   ): draw_dline(dr,x+(           0),y+(side_length/2),1, 1);break;
      case(FrameID::top_right   ): draw_dline(dr,x+(side_length),y+(side_length  ),1,-1);break;


      case(FrameID::top_back    ): draw_dline2(dr,x+(side_length),y+                 0,1, 1);break;
      case(FrameID::top_left    ): draw_dline2(dr,x+(           0),y+(side_length/2)-1,1,-1);break;


      case(FrameID::front_bottom): draw_dline(dr,x+(0),           y+(side_length*2)-(side_length/2)-1,1, 1);break;
      case(FrameID::right_bottom): draw_dline(dr,x+(side_length),y+(side_length*2)                 -1,1,-1);break;

      case(FrameID::front_top   ): draw_dline(dr,x+(           0),y+(side_length/2)+1,1, 1);break;
      case(FrameID::right_top   ): draw_dline(dr,x+(side_length),y+(side_length  )+1,1,-1);break;


      case(FrameID::front_left  ): dr.draw_vline(pixel,x+               0,y+side_length-(side_length/2),side_length);break;
      case(FrameID::front_right ): dr.draw_vline(pixel,x+(side_length)-1,y+side_length                 ,side_length);break;

      case(FrameID::right_left  ): dr.draw_vline(pixel,x+(side_length  )  ,y+side_length                 ,side_length);break;
      case(FrameID::right_right ): dr.draw_vline(pixel,x+(side_length*2)-1,y+side_length-(side_length/2),side_length);break;
      default:;
    }
}


void
draw_frame(Drawer&  dr, int  x, int  y, Dir  dir)
{
  using ID = FrameID;

    switch(dir)
    {
      case(Dir::front):
        draw_frame(dr,x,y,ID::right_left);
//        draw_frame(dr,x,y,ID::top_front );
        break;
      case(Dir::back):
        draw_frame(dr,x,y,ID::right_right);
        draw_frame(dr,x,y,ID::top_back   );
        break;
      case(Dir::left):
        draw_frame(dr,x,y,ID::front_left);
        draw_frame(dr,x,y,ID::top_left  );
        break;
      case(Dir::right):
        draw_frame(dr,x,y,ID::front_right);
//        draw_frame(dr,x,y,ID::top_right  );
        break;
      case(Dir::top):
        draw_frame(dr,x,y,ID::top_front);
        draw_frame(dr,x,y,ID::top_right);
        break;
      case(Dir::bottom):
        draw_frame(dr,x,y,ID::front_bottom);
        draw_frame(dr,x,y,ID::right_bottom);
        break;
      default:;
    }
}



}




