#include<SDL2/SDL.h>
#include"rep.hpp"




namespace{


enum{
  format = SDL_PIXELFORMAT_ABGR8888,
  access = SDL_TEXTUREACCESS_STREAMING,

  zoom_size = 16,

  side_length = 16,
  side_length_ = 16,
  side_length_half = side_length/2,

  window_w = zoom_size*side_length*2,
  window_h = window_w,

};


uint8_t
bmp[side_length*2][side_length*2];


SDL_Window*      window;
SDL_Renderer*  renderer;
SDL_Texture*    texture;

int  pitch;

void*  pixels;


int  looping;


void
draw_point(int  v, int  x, int  y)
{
  bmp[y][x] = v;
}


void
draw_hline(int  v, int  x, int  y, int  l)
{
    while(l--)
    {
      draw_point(v,x++,y);
    }
}


void
draw_vline(int  v, int  x, int  y, int  l)
{
    while(l--)
    {
      draw_point(v,x,y++);
    }
}


void
draw_top(int  pixel)
{
  int  x = 0;
  int  y = 0;

  y += (side_length_/2)-1;

  draw_hline(pixel,x,y,(side_length_*2));


  int  xx = x+2;
  int  yy = y-1;

  int  w = (side_length_*2)-4;
  int  n = (side_length_/2)-1;

    while(n--)
    {
      draw_hline(pixel,xx,yy,w);

      xx += 2;
      yy -= 1;
       w -= 4;
    }


  ++x;
  ++y;

  w = (side_length_*2)-2;
  n = (side_length_/2);

    while(n--)
    {
      draw_hline(pixel,x,y,w);

      x += 2;
      y += 1;
      w -= 4;
    }
}


void
draw_front(int  pixel)
{
  int  x = 0;
  int  y = 0;

  int  h = side_length_;

  y += (side_length_/2);

  draw_vline(pixel,x,y,h);

  x += 1;
  y += 1;

  int  n = ((side_length_/2)-1);

    while(n--)
    {
      draw_vline(pixel,x++,y  ,h);
      draw_vline(pixel,x++,y++,h);
    }


  draw_vline(pixel,x,y,h);
}


void
draw_right(int  pixel)
{
  int  x = 0;
  int  y = 0;

  int  h = side_length_;

  x += side_length_;
  y += side_length_;

  draw_vline(pixel,x,y,h);

  x += 1;
  y -= 1;

  int  n = (side_length_/2)-1;

    while(n--)
    {
      draw_vline(pixel,x++,y  ,h);
      draw_vline(pixel,x++,y--,h);
    }


  draw_vline(pixel,x,y,h);
}


void
draw_dline(int  pixel, int  x, int  y, int  xmod, int  ymod)
{
  draw_point(pixel,x,y);

  x += xmod;
  y += ymod;

    for(int  n = (side_length_/2)-1;  n--;)
    {
      draw_hline(pixel,x,y,2);

      x += xmod*2;
      y += ymod  ;

    }


  draw_point(pixel,x,y);
}


void
draw_dline2(int  pixel, int  x, int  y, int  xmod, int  ymod)
{
    for(int  n = (side_length_/2);  n--;)
    {
      draw_hline(pixel,x,y,2);

      x += xmod*2;
      y += ymod  ;
    }
}


void
draw_cube(SDL_Renderer*  target, int  x, int  y)
{
  SDL_Rect  rect = {0,0,zoom_size,zoom_size};

    rep(yy,side_length*2){
    rep(xx,side_length*2){
      rect.x = zoom_size*xx;
      rect.y = zoom_size*yy;

        switch(bmp[yy][xx])
        {
          case(0): SDL_SetRenderDrawColor(renderer,0x00,0x00,0x00,0xFF);break;
          case(1): SDL_SetRenderDrawColor(renderer,0xFF,0x00,0x00,0xFF);break;
          case(2): SDL_SetRenderDrawColor(renderer,0x00,0xFF,0x00,0xFF);break;
          case(3): SDL_SetRenderDrawColor(renderer,0x00,0x00,0xFF,0xFF);break;
          case(4): SDL_SetRenderDrawColor(renderer,0x7F,0x7F,0x7F,0xFF);break;
        }


      SDL_RenderFillRect(renderer,&rect);
    }}
}


static
void
draw()
{
  SDL_Rect  rect = {0,0,window_w,window_h};

  SDL_SetRenderDrawColor(renderer,0,0,0,0);

  SDL_RenderClear(renderer);

  draw_top(1);
  draw_front(2);
  draw_right(3);


  draw_dline(4,(0),
               (side_length_/2),1,1);

  draw_dline(4,(side_length_),
               (side_length_),1,-1);

/*
  draw_dline(4,(0),
               (side_length_*2)-(side_length_/2)-1,1,1);

  draw_dline(4,(side_length_),
               (side_length_*2)-1,1,-1);



  draw_dline2(4,(0),
                (side_length_/2)-1,1,-1);


  draw_dline2(4,(side_length),
                0,1,1);
*/


  draw_cube(renderer,0,0);

  SDL_RenderPresent(renderer);
}


}




int
main(int  argc, char**  argv)
{
  SDL_Init(SDL_INIT_VIDEO);

  window = SDL_CreateWindow("",0,0,window_w,window_h,0);

  renderer = SDL_CreateRenderer(window,-1,0);

  texture = SDL_CreateTexture(renderer,format,access,window_w,window_h);

  atexit(SDL_Quit);

  looping = true;

  draw();

  static SDL_Event  evt;

    while(looping)
    {
        while(SDL_PollEvent(&evt))
        {
            switch(evt.type)
            {
              case(SDL_WINDOWEVENT):
                  if(evt.window.event == SDL_WINDOWEVENT_EXPOSED)
                  {
                    draw();
                  }
                break;
              case(SDL_QUIT):
                looping = false;
                break;
            }
        }


      SDL_Delay(20);
    }


  SDL_DestroyTexture(texture);
  SDL_DestroyRenderer(renderer);
  SDL_DestroyWindow(window);

  return 0;
}




