#include"heap.hpp"
#include"block_buffer.hpp"
#include"palette.hpp"
#include<algorithm>




Heap  heap_table[heap_table_size];


Point3D  Heap::current_point;


const Space::RenderingParameter
Heap::rendparam(size,size,size);


const int  Heap::width  = 48+font_size*12;
const int  Heap::height = 48;


void
Heap::
draw(Drawer&  dr, void*  data, int  item_index, int  x, int  y)
{
  auto&  h = heap_table[item_index];

  const int  n = (side_length*2);

  dr.draw_rect(0xFF00FFFF,x+4,y+4,n,n);

  transfer(h.thumbnail_,0,0,0,0,dr,x+4,y+4);

  dr.draw_unicode(h.name_.data(),black_color,x+n+8,y+(n/2));
}




Heap::
Heap():
name_(u"new_heap")
{
  std::memset(&elements_,0,sizeof(elements_));

  const int  n = (side_length*2);

  thumbnail_.resize(n,n);
}



      Heap::Element&  Heap::get(int  x, int  y, int  z)      {return elements_[(size*size*z)+(size*y)+x];}
const Heap::Element&  Heap::get(int  x, int  y, int  z) const{return elements_[(size*size*z)+(size*y)+x];}


bool
locked;


void
Heap::
unlock()
{
  locked = false;
}


namespace Flag{
enum
{
    top = 1,
  front = 2,
  right = 4,
};
}


bool
Heap::
process(const Controller&  ctrl, ProcessID  id, int  i)
{
    if(ctrl.test_pressing(Key::shift_flag))
    {
           if(ctrl.test_pressing(Key::up_flag  )){if(current_point.z < (size-1)){current_point.z += 1;}}
      else if(ctrl.test_pressing(Key::down_flag)){if(current_point.z           ){current_point.z -= 1;}}
    }

  else
    {
      const bool  flag = ctrl.test_pressing(Key::p_flag);

           if(ctrl.test_pressing(Key::up_flag   )){if(current_point.y           ){  current_point.y -= 1;  unlock();}}
      else if(ctrl.test_pressing(Key::left_flag )){if(current_point.x           ){  current_point.x -= 1;  unlock();}}
      else if(ctrl.test_pressing(Key::right_flag)){if(current_point.x < (size-1)){  current_point.x += 1;  unlock();}}
      else if(ctrl.test_pressing(Key::down_flag )){if(current_point.y < (size-1)){  current_point.y += 1;  unlock();}}
      else
        if(ctrl.test_pressing(Key::n_flag))
        {
          return false;
        }


        if(flag && !locked)
        {
          auto&  e = get(Heap::current_point.x,Heap::current_point.y,Heap::current_point.z);

            switch(id)
            {
              case(ProcessID::paint_top  ): e.color.top_index   = i;  e.flags |= Flag::top;break;
              case(ProcessID::paint_front): e.color.front_index = i;  e.flags |= Flag::front;break;
              case(ProcessID::paint_right): e.color.right_index = i;  e.flags |= Flag::right;break;

              case(ProcessID::paste):
                e.picture.heap_index = i;
                e.flags = 8;
                break;
              case(ProcessID::erase):
                e.flags = 0;
                break;
            }


          locked = true;
        }
    }


  return true;
}




struct
Heap::
Shell
{
  uint32_t  priority_number;

  int  focus;

  Point2D  point;

  const Element*  element;


  Shell(const Element*  e, Point3D  pt):
  priority_number(pt.x+pt.y+pt.z),
  focus(0),
  point(pt),
  element(e)
  {
    point.x += rendparam.x_offset;
    point.y += rendparam.y_offset;
  }

};


std::vector<Heap::Shell>
buf;


bool
compare(const Heap::Shell&  a, const Heap::Shell&  b)
{
  return(a.priority_number < b.priority_number);
}


void
Heap::
update()
{
  buf.clear();

    rep(z,size){
    rep(y,size){
    rep(x,size){
      auto&  elm = get(x,y,z);

        if(elm.flags)
        {
          buf.emplace_back(&elm,Point3D(x*side_length,
                                        y*side_length,
                                        z*side_length));
        }
    }}}


  std::stable_sort(buf.begin(),buf.end(),compare);

  PixelMap16  tmp(rendparam.width,rendparam.height);

    for(auto&  sh: buf)
    {
      auto&  e = *sh.element;

        if(e.flags == 8)
        {
          auto&  pic = heap_table[e.picture.heap_index];

          transfer(pic.thumbnail_,0,0,0,0,tmp,sh.point.x,sh.point.y);
        }

      else
        {
            if(e.flags&Flag::top  ){cube::fill_top_plane(  tmp,palette_.entries[e.color.top_index  ].pixel,sh.point.x,sh.point.y);}
            if(e.flags&Flag::front){cube::fill_front_plane(tmp,palette_.entries[e.color.front_index].pixel,sh.point.x,sh.point.y);}
            if(e.flags&Flag::right){cube::fill_right_plane(tmp,palette_.entries[e.color.right_index].pixel,sh.point.x,sh.point.y);}
        }
    }


  thumbnail_.fill();

  transform(tmp,thumbnail_);
}




void
Heap::
update(Drawer&  dr, bool  frame)
{
    if(frame)
    {
      dr.draw_rect(0xFFFFFFFF,0,0,rendparam.width,rendparam.height);
    }


  buf.clear();

  bool  found_flag = false;

    rep(z,size){
    rep(y,size){
    rep(x,size){
      auto&  elm = get(x,y,z);

        if(elm.flags)
        {
          buf.emplace_back(&elm,Point3D(x*side_length,
                                        y*side_length,
                                        z*side_length));

            if(!found_flag && (current_point == Point3D(x,y,z)))
            {
              found_flag = true;

              buf.back().focus = 1;
            }
        }
    }}}



    if(!found_flag)
    {
      auto&  current = get(current_point.x,
                           current_point.y,
                           current_point.z);

      buf.emplace_back(&current,Point3D(current_point.x*side_length,
                                        current_point.y*side_length,
                                        current_point.z*side_length));

      buf.back().focus = 1;
    }


  std::stable_sort(buf.begin(),buf.end(),compare);

    for(auto&  sh: buf)
    {
      auto&  e = *sh.element;

        if(e.flags)
        {
            if(e.flags == 8)
            {
              auto&  pic = heap_table[e.picture.heap_index];

              transfer(pic.thumbnail_,0,0,0,0,dr,sh.point.x,sh.point.y);
            }

          else
            {
                if(e.flags&Flag::top  ){cube::fill_top_plane(  dr,palette_.entries[e.color.top_index  ].pixel,sh.point.x,sh.point.y);}
                if(e.flags&Flag::front){cube::fill_front_plane(dr,palette_.entries[e.color.front_index].pixel,sh.point.x,sh.point.y);}
                if(e.flags&Flag::right){cube::fill_right_plane(dr,palette_.entries[e.color.right_index].pixel,sh.point.x,sh.point.y);}
            }
        }


        if(sh.focus)
        {
            rep(i,Dir::END)
            {
              auto  dir = (Dir)i;

              cube::draw_frame(dr,sh.point.x,sh.point.y,dir);
            }
        }
    }


  dr.draw_formatted(0xFFFFFFFF,0,          0,"X %2d",current_point.x);
  dr.draw_formatted(0xFFFFFFFF,0,font_size  ,"Y %2d",current_point.y);
  dr.draw_formatted(0xFFFFFFFF,0,font_size*2,"Z %2d",current_point.z);
}




void
Heap::
load(Stream&  src)
{
  std::memset(&elements_,0,sizeof(elements_));

  name_.clear();

  src.read(name_);

  palette_.load(src);

    for(;;)
    {
      const unsigned int  c = src.get_be16();

        if(!c)
        {
          break;
        }


      const int  i = src.get_be16();

      auto&  e = elements_[i];

      e.flags = (c>>12);

        if(e.flags == 8)
        {
          e.picture.heap_index = (c&0xFFF);
        }

      else
        {
          e.color.top_index   = (c>>8)&0xF;
          e.color.front_index = (c>>4)&0xF;
          e.color.right_index = (c   )&0xF;
        }
    }


  update();
}


void
Heap::
save(Stream&  dst) const
{
  dst.write(name_);

  palette_.save(dst);

    for(auto&  e: elements_)
    {
        if(e.flags)
        {
            if(e.flags == 8)
            {
              dst.put_be16((e.flags<<12)|e.picture.heap_index);
            }

          else
            {
              dst.put_be16((e.flags<<12)|(e.color.top_index<<8)|(e.color.front_index<<4)|e.color.right_index);
            }


          dst.put_be16((&e)-(&elements_[0]));
        }
    }


  dst.put_be16(0);
}




