#ifndef HEAP_HPP_INCLUDED
#define HEAP_HPP_INCLUDED


#include"impr_space.hpp"
#include"cube.hpp"
#include"palette.hpp"
#include"rep.hpp"
#include"fileio.hpp"
#include"controller.hpp"




class Stream;


struct
ColorSpec
{
  uint8_t    top_index;
  uint8_t  front_index;
  uint8_t  right_index;

};


struct
PictureSpec
{
  uint16_t  heap_index;

};


enum class
ProcessID
{
  erase,

  paint_top,
  paint_front,
  paint_right,

  paste,

};


struct
Heap
{
  static constexpr int  size=8;

  struct Shell;

  static Point3D  current_point;

  static const Space::RenderingParameter  rendparam;

  std::u16string  name_;

  Palette  palette_;

  struct Element{
    uint8_t  flags=0;

    union{
      ColorSpec      color;
      PictureSpec  picture;
    };
  };


  Element  elements_[size*size*size];

  PixelMap16  thumbnail_;


  Heap();

        Element&  get(int  x, int  y, int  z)      ;
  const Element&  get(int  x, int  y, int  z) const;

  void  update(                              );
  void  update(Drawer&  dr, bool  frame=false);

  void  render_whole_view();

  const PixelMap16&  update_thumbnail();

  bool  process(const Controller&  ctrl, ProcessID  id, int  i);

  static void  unlock();

  static const int  width;
  static const int  height;

  static void  draw(Drawer&  dr, void*  data, int  item_index, int  x, int  y);


  void  load(Stream&  src)      ;
  void  save(Stream&  dst) const;

};


enum{
  heap_table_size = 80,
};


extern Heap  heap_table[heap_table_size];




#endif




