#include"impr.hpp"




void
Space::RenderingParameter::
make(const Size3D&  size)
{
  const int  side_length_half = (side_length/2);

                 height = (side_length_half*((size.height*2)-2));
  y_offset = height                                         ;


  width   = (side_length     *(size.x_width+size.y_width  ));
  height += (side_length_half*(size.x_width+size.y_width+2));

  x_offset = (width>>1);

    if(size.x_width < size.y_width){x_offset += side_length_half*((size.y_width-size.x_width)-2);}
   else                            {x_offset -= side_length_half*((size.x_width-size.y_width)+2);}
}




Space::
Space(Callback  callback):
callback_(callback)
{
}


Space::
Space(Callback  callback, const Size3D&  array_size, const Size3D&  scan_size):
callback_(callback)
{
  resize(array_size,scan_size);
}


Space::
~Space()
{
  clear();
}




void
Space::
copy(const Space&  rhs)
{
  resize(rhs.array_size_,rhs.scan_size_);

    rep(z,array_size_.height ){
    rep(y,array_size_.y_width){
    rep(x,array_size_.x_width){
      auto  srcobj = rhs.get(x,y,z).get_object();

        if(srcobj)
        {
        }
    }}}
}


Area&
Space::
get(int  i)
{
  return areas_[i];
}


const Area&
Space::
get(int  i) const
{
  return areas_[i];
}




Area&
Space::
get(int  x, int  y, int  z)
{
  return areas_[ (array_size_.x_width*array_size_.y_width*z)
                +(array_size_.x_width                    *y)
                +(                                        x)];
}


const Area&
Space::
get(int  x, int  y, int  z) const
{
  return areas_[ (array_size_.x_width*array_size_.y_width*z)
                +(array_size_.x_width                    *y)
                +(                                        x)];
}


int
Space::
get_area_index(const Area&  area) const
{
  auto  ptr = &area;

  auto  begin_ptr = &*areas_.cbegin();
  auto    end_ptr = &*areas_.cend()  ;

    if((ptr >= begin_ptr) &&
       (ptr <  end_ptr  ))
    {
      return ptr-begin_ptr;
    }


  return -1;
}


void
Space::
insert_object(Object*  obj)
{
    if(!obj->test_flag(Object::move_flag))
    {
      obj->set_flag(Object::move_flag);

      obj->unset_flag(Object::eraseme_flag);

      moving_objects_.push_front(obj);

      obj->set_key(moving_objects_.begin());
    }
}


void
Space::
erase_object(Object*  obj)
{
    if(obj->test_flag(Object::move_flag))
    {
      obj->unset_flag(Object::move_flag);

      moving_objects_.erase(obj->get_key());

      obj->get_area()->update_around_object(*this);
    }
}


void
Space::
release_object(Object*  obj)
{
  erase_object(obj);

  auto&  area = *obj->get_area();

  area.set_object(nullptr);

  area.update_around_object(*this);

  auto  top = area.get_adjoining_area(Dir::top);

    if(top)
    {
      top->try_fall_object(*this);
    }


  callback_(obj,Signal::release);
}


void
Space::
release_objects_all()
{
    for(auto&  area: areas_)
    {
      auto  obj = area.get_object();

        if(obj)
        {
          callback_(obj,Signal::release);
        }
    }


  moving_objects_.clear();
}


const Space::RenderingParameter&  Space::get_view_parameter()       const{return       view_parameter;}
const Space::RenderingParameter&  Space::get_whole_view_parameter() const{return whole_view_parameter;}

const Size3D&  Space::get_size() const{return size_;}

const std::list<Object*>&
Space::
get_moving_objects() const
{
  return moving_objects_;
}




void
Space::
resize(const Size3D&  array_size, const Size3D&  scan_size)
{
  clear();

  array_size_ = array_size;
  scan_size_  =  scan_size;


  size_.x_width = (side_length*array_size_.x_width);
  size_.y_width = (side_length*array_size_.y_width);
  size_.height  = (side_length*array_size_.height );

  view_rect.reset(0,std::min(size_.x_width,side_length*scan_size_.x_width),
                  0,std::min(size_.y_width,side_length*scan_size_.y_width),
                  0,std::min(size_.height ,side_length*scan_size_.height ));


  scan_size_.x_width = (view_rect.x_width/side_length);
  scan_size_.y_width = (view_rect.y_width/side_length);
  scan_size_.height  = (view_rect.height /side_length);

        view_parameter.make(scan_size);
  whole_view_parameter.make(array_size);


  areas_.resize(array_size[1]);

    rep(z,array_size_.height){
    rep(y,array_size_.y_width){
    rep(x,array_size_.x_width){
      auto&  area = get(x,y,z);

      area.set_point(x,y,z);

      area.set_object(nullptr);

      area.set_adjoining_area(Dir::bottom,((z                          )? &get(x  ,y  ,z-1):nullptr));
      area.set_adjoining_area(Dir::top   ,((z < (array_size_.height -1))? &get(x  ,y  ,z+1):nullptr));
      area.set_adjoining_area(Dir::back  ,((y                          )? &get(x  ,y-1,z  ):nullptr));
      area.set_adjoining_area(Dir::front ,((y < (array_size_.y_width-1))? &get(x  ,y+1,z  ):nullptr));
      area.set_adjoining_area(Dir::left  ,((x                          )? &get(x-1,y  ,z  ):nullptr));
      area.set_adjoining_area(Dir::right ,((x < (array_size_.x_width-1))? &get(x+1,y  ,z  ):nullptr));
    }}}
}




void
Space::
clear()
{
  release_objects_all();

  areas_.clear();
}




void
Space::
move()
{
    if(moving_objects_.empty())
    {
      return;
    }


  auto   it = moving_objects_.begin();
  auto  end = moving_objects_.end();

    while(it != end)
    {
      auto  obj = *it++;

      obj->process(*this);

        if(obj->test_flag(Object::releaseme_flag))
        {
          release_object(obj);
        }

      else
        if(obj->test_flag(Object::eraseme_flag))
        {
          erase_object(obj);
        }
    }
}




