#ifndef IMPR_SPACE_HPP_INCLUDED
#define IMPR_SPACE_HPP_INCLUDED


#include"impr_rect.hpp"
#include"impr_object.hpp"
#include"impr_area.hpp"
#include"impr_shell.hpp"
#include<vector>




class
Space
{
public:
  struct RenderingParameter{
    int  width ;
    int  height;

    int  x_offset;
    int  y_offset;

    static constexpr int  calculate_width(int  x_width, int  y_width)
    {
      return(side_length*(x_width+y_width));
    }

    static constexpr int  calculate_height(int  x_width, int  y_width, int  height)
    {
      return(calculate_y_offset(height)+((side_length/2)*(x_width+y_width+2)));
    }

    static constexpr int  calculate_x_offset(int  x_width, int  y_width)
    {
      return((calculate_width(x_width,y_width)/2)+((x_width < y_width)?  (side_length/2)*((y_width-x_width)-2)
                                                                      : -(side_length/2)*((x_width-y_width)+2)));
    }

    static constexpr int  calculate_y_offset(int  height)
    {
      return((side_length/2)*((height*2)-2));
    }

    constexpr RenderingParameter():
     width(0),
    height(0),
    x_offset(0),
    y_offset(0)
    {
    }

    constexpr RenderingParameter(int  x_width, int  y_width, int  height):
     width(calculate_width(x_width,y_width)),
    height(calculate_height(x_width,y_width,height)),
    x_offset(calculate_x_offset(x_width,y_width)),
    y_offset(calculate_y_offset(height))
    {
    }


    void  make(const Size3D&  size);

  };


private:
  Size3D  array_size_;
  Size3D   scan_size_;

  RenderingParameter        view_parameter;
  RenderingParameter  whole_view_parameter;

  Size3D  size_;

  Rect3D  view_rect;

  std::list<Object*>  moving_objects_;

  std::vector<Area>  areas_;

public:
  enum class Signal{
    release,
    update,
    duplicate,
  };


  using Callback = void  (*)(Object*  obj, Signal  sig);

  Callback  const callback_;

   Space(Callback  callback                                                     );
   Space(Callback  callback, const Size3D&  array_size, const Size3D&  scan_size);
  ~Space(                                                                       );


        Area&  get(int  i)      ;
  const Area&  get(int  i) const;

        Area&  get(int  x, int  y, int  z)      ;
  const Area&  get(int  x, int  y, int  z) const;

  int  get_area_index(const Area&  area) const;

  void  insert_object(Object*  obj);
  void   erase_object(Object*  obj);
  void  release_object(Object*  obj);
  void  release_objects_all();

  void  copy(const Space&  rhs);

  void  move();

  void  resize(const Size3D&  array_size, const Size3D&  scan_size);
  void  clear();


  const std::list<Object*>&  get_moving_objects() const;

  const Rect3D&  get_view_rect() const;
  void  set_view_point(int  x, int  y, int  z);

  const RenderingParameter&        get_view_parameter() const;
  const RenderingParameter&  get_whole_view_parameter() const;

  const Size3D&  get_size() const;


  void  move_view_left(  int  dist);
  void  move_view_right( int  dist);
  void  move_view_back(  int  dist);
  void  move_view_front( int  dist);
  void  move_view_top(   int  dist);
  void  move_view_bottom(int  dist);


  void  collect_visible_objects(    std::vector<Shell>&  buf) const;
  void  collect_visible_objects_all(std::vector<Shell>&  buf) const;

  void  collect_objects_all(std::vector<const Object*>&  buf) const;

};




#endif




