#include"krtdb.hpp"




namespace krtdb{




Record::
Record(Table&  table, UniqueNumber  number, size_t  n):
table_(table),
number_(number),
values_(n)
{
}


Record::
Record(Table&  table, UniqueNumber  number, std::initializer_list<Value>  values):
table_(table),
number_(number)
{
  *this = values;
}


Record::
Record(Table&  table, UniqueNumber  number, Stream&  s):
table_(table),
number_(number)
{
  read(s);
}


Record::
Record(Record&&  rhs):
table_(rhs.table_),
number_(rhs.number_),
values_(std::move(rhs.values_))
{
}




Pointer
Record::
get_pointer() const
{
  auto  p = table_.get_pointer();

  p.record_number_ = number_;

  return p;
}


void
Record::
clear()
{
    if(values_.size() == table_.get_def().headers_.size())
    {
      Value*  v = values_.data();

        for(auto&  hdr: table_.get_def().headers_)
        {
            if((hdr.valuekind_ == ValueKind::string) ||
               (hdr.valuekind_ == ValueKind::binary))
            {
              v->bs_.destroy_buffer();
            }


          ++v;
        }


      values_.clear();
    }
}


void
Record::
operator=(std::initializer_list<Value>  values)
{
  values_.clear();

    for(auto&  src: values)
    {
      values_.emplace_back(std::move(src));
    }
}




void
Record::
read(Stream&  s)
{
  const auto  n = table_.get_def().headers_.size();

  values_.resize(n);

    for(int  i = 0;  i < n;  ++i)
    {
      auto&  hdr = table_.get_def().headers_[i];

      values_[i].read(hdr.valuekind_,s);
    }
}


void
Record::
write(Stream&  s) const
{
  const auto  n = table_.get_def().headers_.size();

    for(int  i = 0;  i < n;  ++i)
    {
      auto&  hdr = table_.get_def().headers_[i];

        if(hdr.valuekind_ != ValueKind::null)
        {
          values_[i].write(hdr.valuekind_,s);
        }
    }
}


void
Record::
print() const
{
  const auto  n = table_.get_def().headers_.size();

    for(int  i = 0;  i < n;  ++i)
    {
      auto&  hdr = table_.get_def().headers_[i];

      values_[i].print(hdr.valuekind_);

      printf(", ");
    }
}




}




