#include"krtdb.hpp"




namespace krtdb{




Storage::
Storage():
last_table_number_(1)
{
}




Table&
Storage::
new_table(std::string&&  name, const TableDef&  def)
{
  return new_table(std::move(name),TableDef(def));
}


Table&
Storage::
new_table(std::string&&  name, TableDef&&  def)
{
  auto  n = get_unused_number(tables_,last_table_number_);

  auto  p = tables_.emplace(n,Table(*this,n,name,std::move(def)));

  auto&  t = p.first->second;

  namelinks_.emplace(std::move(name),&t);

  return t;
}


bool
Storage::
test_table(UniqueNumber  n) const
{
  return(tables_.find(n) != tables_.cend());
}


bool
Storage::
test_table(const std::string&  name) const
{
  return(namelinks_.find(name) != namelinks_.cend());
}




Table&
Storage::
get_table(UniqueNumber  n)
{
  return tables_.at(n);
}


const Table&
Storage::
get_table(UniqueNumber  n) const
{
  return tables_.at(n);
}


Table&
Storage::
get_table(const std::string&  name)
{
  return *namelinks_.at(name);
}


const Table&
Storage::
get_table(const std::string&  name) const
{
  return *namelinks_.at(name);
}




enum{
  storage_version = 0,
    table_version = 0,

};


void
Storage::
read(Stream&  s)
{
  tables_.clear();

  last_table_number_ = 1;

  auto  stover = s.get_uvlint();
  auto  tblver = s.get_uvlint();

  auto  count = s.get_uvlint();

    while(count--)
    {
      auto  n = s.get_uvlint();

      last_table_number_ = std::max(last_table_number_,n);

      auto  p = tables_.emplace(n,Table(*this,n));

      p.first->second.read(s);

      namelinks_.emplace(p.first->second.get_name(),&p.first->second);
    }


  ++last_table_number_;
}


void
Storage::
write(Stream&  s) const
{
  s.put_uvlint(storage_version);
  s.put_uvlint(table_version);

  s.put_uvlint(tables_.size());

    for(auto&  p: tables_)
    {
      s.put_uvlint(p.first);

      p.second.write(s);
    }
}


void
Storage::
print() const
{
  printf("%d tables.\n",tables_.size());

    for(auto&  p: tables_)
    {
      p.second.print();

      printf("\n");
    }
}




}




