#include"krtdb.hpp"




namespace krtdb{




Table::
Table(Storage&  storage, UniqueNumber  number):
storage_(storage),
number_(number),
last_record_number_(1)
{
}


Table::
Table(Storage&  storage, UniqueNumber  number, const std::string&  name, TableDef&&  def):
storage_(storage),
number_(number),
name_(name),
def_(std::move(def)),
last_record_number_(1)
{
}


Table::
Table(Table&&  rhs):
storage_(rhs.storage_),
number_(rhs.number_),
name_(std::move(rhs.name_)),
def_(std::move(rhs.def_)),
last_record_number_(rhs.last_record_number_),
records_(std::move(rhs.records_))
{
}


Table::
~Table()
{
  clear();
}




void
Table::
clear()
{
    for(auto&  p: records_)
    {
      p.second.clear();
    }


  records_.clear();
}


const std::string&
Table::
get_name() const
{
  return name_;
}


const std::unordered_map<UniqueNumber,Record>&
Table::
get_records() const
{
  return records_;
}


const TableDef&
Table::
get_def() const
{
  return def_;
}




Pointer
Table::
get_pointer() const
{
  return Pointer(number_);
}


Record&
Table::
new_record()
{
  auto  n = get_unused_number(records_,last_record_number_);

  auto  p = records_.emplace(n,Record(*this,n,def_.headers_.size()));

  return p.first->second;
}


Record&
Table::
new_record(std::initializer_list<Value>  values)
{
  auto  n = get_unused_number(records_,last_record_number_);

  auto  p = records_.emplace(n,Record(*this,n,values));

  return p.first->second;
}


void
Table::
delete_record(UniqueNumber  n)
{
  records_.erase(n);
}


bool
Table::
test_record(UniqueNumber  n) const
{
  return(records_.find(n) != records_.cend());
}


Record&
Table::
get_record(UniqueNumber  n)
{
  return records_.at(n);
}


const Record&
Table::
get_record(UniqueNumber  n) const
{
  return records_.at(n);
}




void
Table::
read(Stream&  s)
{
     name_.clear();
      def_.clear();
  records_.clear();

  last_record_number_ = 1;


  auto  l = s.get_uvlint();

    while(l--)
    {
      name_.push_back(s.get());
    }


  def_.read(s);

  auto  count = s.get_uvlint();

    while(count--)
    {
      auto  n = s.get_uvlint();

      last_record_number_ = std::max(last_record_number_,n);

      records_.emplace(n,Record(*this,n,s));
    }


  ++last_record_number_;
}


void
Table::
write(Stream&  s) const
{
  s.put_uvlint(name_.size());

  s.write(name_.data(),name_.size());


  def_.write(s);

  s.put_uvlint(records_.size());

    for(auto&  p: records_)
    {
      s.put_uvlint(p.first);

      p.second.write(s);
    }
}


void
Table::
print() const
{
  printf("<%s>\n",name_.data());

  printf("%d records.\n",records_.size());

  def_.print();

  printf("\n");

    for(auto&  p: records_)
    {
      p.second.print();

      printf("\n");
    }
}


}




