#ifndef KRTDB_TABLE_HPP_INCLUDED
#define KRTDB_TABLE_HPP_INCLUDED


#include"krtdb_record.hpp"
#include<unordered_map>
#include<limits>


namespace krtdb{


class Storage;


template<typename  M, typename  N>
N
get_unused_number(const M&  m, N&  n)
{
    if(n == std::numeric_limits<N>::max())
    {
      throw 0;
    }


  const auto  end = m.cend();

    for(;;)
    {
      const N  t = n++;

        if(m.find(t) == end)
        {
          return t;
        }
    }


  return 0;
}




class
Table
{
  Storage&  storage_;

  UniqueNumber  number_;

  UniqueNumber  last_record_number_;

  std::string  name_;

  TableDef  def_;

  std::unordered_map<UniqueNumber,Record>  records_;

public:
  Table(Storage&  storage, UniqueNumber  number                                           );
  Table(Storage&  storage, UniqueNumber  number, const std::string&  name, TableDef&&  def);
  Table(Table&&  rhs);
  ~Table();


  void  clear();

  const std::string&  get_name() const;

  const std::unordered_map<UniqueNumber,Record>&  get_records() const;

  const TableDef&  get_def() const;

  Pointer  get_pointer() const;

  Record&  new_record(                                    );
  Record&  new_record(std::initializer_list<Value>  values);


  void  delete_record(UniqueNumber  n);

  bool  test_record(UniqueNumber  n) const;

        Record&  get_record(UniqueNumber  n)      ;
  const Record&  get_record(UniqueNumber  n) const;

  void   read(Stream&  s)      ;
  void  write(Stream&  s) const;

  void  print() const;

};




}




#endif




