#include"krtdb.hpp"




namespace krtdb{




TableDef::
TableDef(std::initializer_list<Header>  ls):
headers_(ls)
{
}


TableDef::
TableDef(const TableDef&   rhs):
headers_((rhs.headers_))
{
}


TableDef::
TableDef(TableDef&&  rhs):
headers_(std::move(rhs.headers_))
{
}




void
TableDef::
clear()
{
  headers_.clear();
}


int
TableDef::
find(const std::string&  name) const
{
  const int  n = headers_.size();

    for(int  i = 0;  i < n;  ++i)
    {
        if(headers_[i].name_ == name)
        {
          return i;
        }
    }


  return -1;
}




void
TableDef::
read(Stream&  s)
{
  headers_.clear();

  revision_number_ = s.get_uvlint();

  auto  count = s.get_uvlint();

    while(count--)
    {
      std::string  name;

      auto  l = s.get_uvlint();

        while(l--)
        {
          name.push_back(s.get());
        }


      auto  valuekind = (ValueKind)s.get_uvlint();

      headers_.emplace_back(std::move(name),valuekind);
    }
}


void
TableDef::
write(Stream&  s) const
{
  s.put_uvlint(revision_number_);
  s.put_uvlint(headers_.size());

    for(auto&  hdr: headers_)
    {
      s.put_uvlint(hdr.name_.size());

      s.write(hdr.name_.data(),hdr.name_.size());


      s.put_uvlint((int)hdr.valuekind_);
    }
}


void
TableDef::
print() const
{
  printf("rev.%d\n",revision_number_);

    for(auto&  hdr: headers_)
    {
      printf("[%s \"%s\"]",get_value_kind_string(hdr.valuekind_),hdr.name_.data());
    }
}


}




