#include"krtdb.hpp"




namespace krtdb{




Value::
Value()
{
  bs_.buffer_ = nullptr;
}


Value::
Value(int32_t  i): i_(i)
{
}


Value::
Value(uint32_t  u): u_(u)
{
}


Value::
Value(const char*  s)
{
  bs_.create_buffer(std::strlen(s),s);
}


Value::
Value(const std::string&  s)
{
  bs_.create_buffer(s.size(),s.data());
}


Value::
Value(ByteString  bs): bs_(bs)
{
}


Value::
Value(const void*  data, size_t  size)
{
  bs_.create_buffer(size,data);
}


Value::
Value(void*  data, size_t  size)
{
  bs_.buffer_ = (uint8_t*)data;
  bs_.size_   =           size;
}


Value::
Value(Pointer  pointer): pointer_(pointer)
{
}




void
Value::
write(ValueKind  kind, Stream&  s) const
{
    switch(kind)
    {
      case(ValueKind::bool_   ): s.put(i_? 1:0);break;
      case(ValueKind::int8_   ): s.put(i_);break;
      case(ValueKind::uint8_  ): s.put(u_);break;
      case(ValueKind::int16_  ): s.put_be16(i_);break;
      case(ValueKind::uint16_ ): s.put_be16(u_);break;
      case(ValueKind::int32_  ): s.put_be32(i_);break;
      case(ValueKind::uint32_ ): s.put_be32(u_);break;

      case(ValueKind::vlint_):
        s.put_vlint(i_);
        break;
      case(ValueKind::uvlint_):
        s.put_uvlint(u_);
        break;
      case(ValueKind::string):
      case(ValueKind::binary):
        s.put_uvlint(bs_.size_);
        s.write(bs_.buffer_,bs_.size_);
        break;
      case(ValueKind::table):
        s.put_uvlint(pointer_.table_number_);
        break;
      case(ValueKind::record):
        s.put_uvlint(pointer_.table_number_);
        s.put_uvlint(pointer_.record_number_);
        break;
      default:;
    }
}


void
Value::
read(ValueKind  kind, Stream&  s)
{
    switch(kind)
    {
      case(ValueKind::bool_   ): i_ = s.get();break;
      case(ValueKind::int8_   ): i_ = s.get();break;
      case(ValueKind::uint8_  ): u_ = s.get();break;
      case(ValueKind::int16_  ): i_ = s.get_be16();break;
      case(ValueKind::uint16_ ): u_ = s.get_be16();break;
      case(ValueKind::int32_  ): i_ = s.get_be32();break;
      case(ValueKind::uint32_ ): u_ = s.get_be32();break;

      case(ValueKind::vlint_):
        i_ = s.get_vlint();
        break;
      case(ValueKind::uvlint_):
        u_ = s.get_uvlint();
        break;
      case(ValueKind::string):
      case(ValueKind::binary):
        bs_.create_buffer(s.get_uvlint());

        s.read(bs_.buffer_,bs_.size_);
        break;
      case(ValueKind::table):
        pointer_.table_number_ = s.get_uvlint();
        break;
      case(ValueKind::record):
        pointer_.table_number_  = s.get_uvlint();
        pointer_.record_number_ = s.get_uvlint();
        break;
      default:;
    }
}


void
Value::
print(ValueKind  kind) const
{
    switch(kind)
    {
      case(ValueKind::bool_):printf("%s",i_? "true":"false");break;

      case(ValueKind::int8_ ):
      case(ValueKind::int16_):
      case(ValueKind::int32_):
      case(ValueKind::vlint_): printf("%d",i_);break;

      case(ValueKind::uint8_ ):
      case(ValueKind::uint16_):
      case(ValueKind::uint32_):
      case(ValueKind::uvlint_): printf("%u",u_);break;

      case(ValueKind::string): printf("%s",bs_.buffer_);break;
      case(ValueKind::binary): printf("...%d bytes",bs_.size_);break;
      case(ValueKind::table ): printf("#%u",pointer_.table_number_);break;
      case(ValueKind::record): printf("#%u #%u",pointer_.table_number_,pointer_.record_number_);break;
      default:;
        printf("**unknown**");
    }
}


}




