#include"block_buffer.hpp"
#include"heap.hpp"
#include"widget.d/flexmenu.hpp"
#include"widget.d/lettermenu.hpp"
#include"widget.d/frame.hpp"
#include"video.d/video.hpp"
#include"audio.d/audio.hpp"




Controller
ctrl;


namespace{


enum class Task
{
  choose_topmenuitem,
  choose_color,
  modify_colorelement,

  choose_heap,
  choose_heap_for_paste,
  make_heap_name,

  choose_mkhpmenuitem,
  paint_element,
  paste

} task;


ProcessID
procid;


int
proc_i;


int
display_timer;


std::u16string
display_message;


FlexMenu
top_menu({u"ヒープを作成",u"すべて保存"});


LetterMenu
letter_menu;


FlexMenu  heap_menu(8,10,Heap::width,Heap::height,Heap::draw,nullptr);

FlexMenu  heap_menu2(8,10,Heap::width,Heap::height,Heap::draw,nullptr);

FlexMenu  mkhp_menu({u"頂面を塗る",
                     u"前面を塗る",
                     u"右面を塗る",
                     u"消す",
                    });


Frame    topmenu_frame;
Frame  paintmenu_frame;

Frame   heapmenu_frame;
Frame   heapmenu2_frame;


FlexMenu         color_menu(16,1,ColorEntry::width,ColorEntry::height,     ColorEntry::draw,nullptr);
FlexMenu  colorelement_menu(3,1,ColorElement::width,ColorElement::height,ColorElement::draw,nullptr);

Heap*
current_heap = heap_table;

ColorEntry*
current_color = heap_table->palette_.entries;




void
save_heaps_all()
{
  FileStream  f(fopen("test.dat","wb"));

  f.put(0);
  f.put_be16(heap_table_size);

    for(auto&  hp: heap_table)
    {
      hp.save(f);
    }


  display_timer = 32;

  display_message = u"SAVED";
}


void
load_heaps()
{
  auto  f = fopen("test.dat","rb");

    if(f)
    {
      FileStream  fs(f);

      auto  ver = fs.get();

      int  n = fs.get_be16();

      n = std::min(n,(int)heap_table_size);

      Heap*  dst = heap_table;

        while(n--)
        {
          dst++->load(fs);
        }
    }
}


void
choose_topmenuitem()
{
    switch(top_menu.process(ctrl))
    {
      case(FlexMenu::Action::decide):
          switch(top_menu.get_item_index())
          {
            case(0):
              task = Task::choose_heap;
              break;
            case(1):
              save_heaps_all();
              break;
          }
        break;
    }
}


void
modify_colorelement()
{
  auto&  ent = *static_cast<ColorEntry*>(colorelement_menu.get_data());

  int  rgb[] = {to_r4(ent.pixel),
                to_g4(ent.pixel),
                to_b4(ent.pixel)};

  int&  v = rgb[colorelement_menu.get_item_index()];

    switch(colorelement_menu.process(ctrl))
    {
      case(FlexMenu::Action::decide):
      case(FlexMenu::Action::cancel):
        task = Task::choose_color;

        proc_i = color_menu.get_item_index();
        colorelement_menu.set_cursor(false,false);
        break;
      case(FlexMenu::Action::left):
          if(v)
          {
            --v;
          }
        break;
      case(FlexMenu::Action::right):
          if(v < 15)
          {
            ++v;
          }
        break;
    }


  ent.pixel = to_pixel16(rgb[0],rgb[1],rgb[2]);
}


void
choose_color()
{
    switch(color_menu.process(ctrl))
    {
      case(FlexMenu::Action::decide):
        task = Task::modify_colorelement;

        current_color = &current_heap->palette_.entries[color_menu.get_item_index()];

        colorelement_menu.set_data(current_color);

        colorelement_menu.set_cursor(true,true);
        break;
      case(FlexMenu::Action::cancel):
        task = Task::choose_heap;

        current_heap->update();
        break;
      case(FlexMenu::Action::right):
        task = Task::choose_mkhpmenuitem;

        color_menu.set_cursor(false,true);
        mkhp_menu.set_cursor(true,true);
        break;
      case(FlexMenu::Action::up):
      case(FlexMenu::Action::down):
        current_color = &current_heap->palette_.entries[color_menu.get_item_index()];

        colorelement_menu.set_data(current_color);
        break;
      case(FlexMenu::Action::change):
        task = Task::choose_heap_for_paste;
        break;
    }
}


void
choose_heap()
{
    switch(heap_menu.process(ctrl))
    {
      case(FlexMenu::Action::decide):
        task = Task::choose_color;

        color_menu.set_cursor(true,true);
        mkhp_menu.set_cursor(false,false);
        colorelement_menu.set_cursor(false,false);

        current_heap = &heap_table[heap_menu.get_item_index()];
        current_color = current_heap->palette_.entries;

        color_menu.reset_item_index();
        color_menu.set_data(&current_heap->palette_);
        break;
      case(FlexMenu::Action::cancel):
        task = Task::choose_topmenuitem;
        break;
      case(FlexMenu::Action::start):
        task = Task::make_heap_name;

        current_heap = &heap_table[heap_menu.get_item_index()];

        letter_menu.reset_buffer(current_heap->name_);
        break;
    }
}


void
choose_heap_for_paste()
{
    switch(heap_menu2.process(ctrl))
    {
      case(FlexMenu::Action::decide):
        task = Task::paste;

        proc_i = heap_menu2.get_item_index();
        break;
      case(FlexMenu::Action::cancel):
      case(FlexMenu::Action::change):
        task = Task::choose_color;
        color_menu.set_cursor(true,true);
        mkhp_menu.set_cursor(false,false);
        break;
    }
}


void
make_heap_name()
{
    switch(letter_menu.process(ctrl))
    {
      case(LetterMenu::Action::finish):
        task = Task::choose_heap;

        current_heap->name_ = letter_menu.get_buffer();
        break;
    }
}


void
choose_mkhpmenuitem()
{
    switch(mkhp_menu.process(ctrl))
    {
      case(FlexMenu::Action::decide):
          switch(mkhp_menu.get_item_index())
          {
            case(0):
              task = Task::paint_element;
              procid = ProcessID::paint_top;
              proc_i = color_menu.get_item_index();
              break;
            case(1):
              task = Task::paint_element;
              procid = ProcessID::paint_front;
              proc_i = color_menu.get_item_index();
              break;
            case(2):
              task = Task::paint_element;
              procid = ProcessID::paint_right;
              proc_i = color_menu.get_item_index();
              break;
            case(3):
              task = Task::paint_element;
              procid = ProcessID::erase;
             break;
          }
        break;
      case(FlexMenu::Action::cancel):
        task = Task::choose_heap;

        break;
      case(FlexMenu::Action::left):
        task = Task::choose_color;
        color_menu.set_cursor(true,true);
        mkhp_menu.set_cursor(false,false);
        break;
      case(FlexMenu::Action::change):
        task = Task::choose_heap_for_paste;
        break;
    }
}


void
paint_element()
{
    if(ctrl.is_pressing_any())
    {
        if(!current_heap->process(ctrl,procid,proc_i))
        {
          task = Task::choose_mkhpmenuitem;

          current_heap->update();
        }
    }

  else
    {
      Heap::unlock();
    }
}


void
paste()
{
    if(ctrl.is_pressing_any())
    {
        if(!current_heap->process(ctrl,ProcessID::paste,proc_i))
        {
          task = Task::choose_heap_for_paste;

          current_heap->update();
        }
    }

  else
    {
      Heap::unlock();
    }
}


}




void
process(video::Device*  video_device)
{
  static uint32_t  last = 0;

  auto  now = SDL_GetTicks();

    if(now >= last+120)
    {
      last = now;

        switch(task)
        {
          case(Task::choose_topmenuitem ): choose_topmenuitem();   break;
          case(Task::choose_color       ): choose_color();         break;
          case(Task::modify_colorelement): modify_colorelement();  break;
          case(Task::choose_heap        ): choose_heap();          break;
          case(Task::choose_heap_for_paste): choose_heap_for_paste();          break;
          case(Task::choose_mkhpmenuitem): choose_mkhpmenuitem();            break;
          case(Task::make_heap_name     ): make_heap_name();       break;
          case(Task::paint_element      ): paint_element();       break;
          case(Task::paste              ): paste();       break;
        }
    }


  video_device->lock();

  auto  dr = video_device->get_drawer();

    switch(task)
    {
      case(Task::choose_topmenuitem):
        topmenu_frame.draw(dr);
        break;
      case(Task::choose_color):
        paintmenu_frame.draw(dr);
        current_heap->update(dr);
        break;
      case(Task::modify_colorelement):
        paintmenu_frame.draw(dr);
        current_heap->update(dr);
        break;
      case(Task::choose_heap):
        heapmenu_frame.draw(dr);
        break;
      case(Task::choose_heap_for_paste):
        heapmenu2_frame.draw(dr);
        current_heap->update(dr);
        break;
      case(Task::choose_mkhpmenuitem):
        paintmenu_frame.draw(dr);
        current_heap->update(dr);
        break;
      case(Task::paint_element):
        paintmenu_frame.draw(dr);
        current_heap->update(dr,true);
        break;
      case(Task::paste):
        heapmenu2_frame.draw(dr);
        current_heap->update(dr,true);
        break;
      case(Task::make_heap_name):
        letter_menu.draw(dr);
        break;
    }


    if(display_timer)
    {
      --display_timer;

      dr.fill_rect(                          0xFFFFFFFF,0,480-font_h-8,font_w*(display_message.size()+1)+8,font_h+8);
      dr.draw_unicode(display_message.data(),0x00000000,4,480-font_h-4                                             );
    }


  video_device->unlock();

  ctrl.clear();
}




void
initialize()
{
  task = Task::choose_topmenuitem;

  top_menu.set_cursor(true,true);
  heap_menu.set_cursor(true,true);
  heap_menu2.set_cursor(true,true);

    topmenu_frame.relative_point.set(0,0);
  paintmenu_frame.relative_point.set(Heap::rendparam.width,0);
   heapmenu_frame.relative_point.set(0,0);
   heapmenu2_frame.relative_point.set(Heap::rendparam.width,0);

    topmenu_frame.reset(u"TOP MENU",{&top_menu},16);
  paintmenu_frame.reset(u"!面を塗る",{&color_menu,&mkhp_menu},16);
   heapmenu_frame.reset(u"ヒープを作成",{&heap_menu},16);
   heapmenu2_frame.reset(u"!サムネイルを貼る",{&heap_menu2},16);


  paintmenu_frame.insert(&colorelement_menu,120,200);
  paintmenu_frame.cleanup();

  colorelement_menu.set_data(current_color);

  load_heaps();
}




