#include"stream.hpp"
#include<random>
#include<limits>




template<typename  INT, typename  PUT, typename  GET>
bool
test_stream_int_rw(Stream&  m, INT  v, PUT  put_fn, GET  get_fn)
{
  m.set_pos(0);

  (m.*put_fn)(v);

  m.set_pos(0);

  auto  res = (m.*get_fn)();

  return(v == res);
}


template<typename  INT, typename  PUT, typename  GET>
bool
test_stream_random_int_rw(Stream&  m, bool  vl, PUT  put_fn, GET  get_fn)
{
  printf("[%ssigned %dbit %sinteger test] ",std::numeric_limits<INT>::is_signed? "  ":"un",sizeof(INT)*8,vl? "variable-length-":"");


  std::random_device  seed;

  std::default_random_engine  engine(seed());

  std::uniform_int_distribution<INT>  dist(std::numeric_limits<INT>::min(),
                                           std::numeric_limits<INT>::max());

    for(int  i = 128;  i;  --i)
    {
      const auto  v = dist(engine);

      fflush(stdout);

        if(!test_stream_int_rw(m,v,put_fn,get_fn))
        {
          printf("failed.\n");

          return false;
        }
    }


  printf("passed.\n");

  return true;
}


void
test(Stream&  m)
{
    if(!test_stream_random_int_rw<uint16_t>(m,false,&Stream::put_be16,&Stream::get_be16) ||
       !test_stream_random_int_rw<uint32_t>(m,false,&Stream::put_be32,&Stream::get_be32) ||
       !test_stream_random_int_rw<  int8_t>(m, true,&Stream::put_uvlint,&Stream::get_uvlint) ||
       !test_stream_random_int_rw< uint8_t>(m, true,&Stream::put_uvlint,&Stream::get_uvlint) ||
       !test_stream_random_int_rw< int16_t>(m, true,&Stream::put_uvlint,&Stream::get_uvlint) ||
       !test_stream_random_int_rw<uint16_t>(m, true,&Stream::put_uvlint,&Stream::get_uvlint) ||
       !test_stream_random_int_rw< int32_t>(m, true,&Stream::put_uvlint,&Stream::get_uvlint) ||
       !test_stream_random_int_rw<uint32_t>(m, true,&Stream::put_uvlint,&Stream::get_uvlint))
    {
      printf("\ntest failed.\n");
    }

  else
    {
      printf("\nall tests passed.\n");
    }


  printf("\n");
}


int
main(int  argc, char**  argv)
{
  MemoryStream  m;

  test(m);


  FileStream  f(tmpfile());

  test(f);

  return 0;
}




