#include"video.hpp"




#ifndef rep
#define rep(v,n)  for(int  v = 0;  v < (n);  v += 1)
#endif


namespace video{




Device::
Device(const char*  title, int  x, int  y, int  w, int  h):
pixels(nullptr),
texture(nullptr)
{
  SDL_InitSubSystem(SDL_INIT_VIDEO);

  rect.x = 0;
  rect.y = 0;
  rect.w = w;
  rect.h = h;

  locked = false;

  window = SDL_CreateWindow(title,x,y,w,h,0);
  renderer = SDL_CreateRenderer(window,-1,0);

  SDL_RendererInfo  info;

  SDL_GetRendererInfo(renderer,&info);

  printf("[usable formats]\n");

    for(int  i = 0;  i < info.num_texture_formats;  ++i)
    {
      printf("*%s\n",SDL_GetPixelFormatName(info.texture_formats[i]));
    }


  auto  format = info.texture_formats[0];

  texture = SDL_CreateTexture(renderer,format,SDL_TEXTUREACCESS_STREAMING,w,h);

  reset_color_table(format);
}


Device::
~Device()
{
  SDL_DestroyTexture(texture);
  SDL_DestroyRenderer(renderer);
  SDL_DestroyWindow(window);

  SDL_QuitSubSystem(SDL_INIT_VIDEO);
}




void
Device::
lock()
{
  SDL_LockTexture(texture,nullptr,&pixels,&pitch);

  locked = true;

  SDL_memset(pixels,0,(pitch*rect.h));
}


void
Device::
unlock()
{
  SDL_UnlockTexture(texture);

  locked = false;

  SDL_RenderCopy(renderer,texture,&rect,&rect);

  SDL_RenderPresent(renderer);
}


bool
Device::
test_locked() const
{
  return locked;
}


int  Device::get_width()  const{return rect.w;}
int  Device::get_height() const{return rect.h;}


Drawer
Device::get_drawer()
{
  return Drawer((uint32_t*)pixels,rect.w,rect.h,pitch);
}




void
Device::
transfer(Drawer&  src_dr, int  src_x, int  src_y, int  dst_x, int  dst_y)
{
/*
  auto  dst_dr = get_drawer();

  const int  w = std::min(rect.w,src_dr.get_width() );
  const int  h = std::min(rect.h,src_dr.get_height());

    rep(y,h)
    {
      auto  dst_p = dst_dr.get(dst_x,dst_y++);
      auto  src_p = src_dr.get(src_x,src_y++);

        rep(x,w)
        {
          *dst_p.ptr32++ =
          *src_p.ptr32++;
        }
    }
*/
}




}




