#include"drawer.hpp"
#include<vector>
#include<libpng/png.h>




#define rep(v,n)  for(int  v = 0;  v < (n);  v += 1)


constexpr int
clamp(int  i, int  min, int  max)
{
  return((i <= min)? min:
         (i >= max)? max:i);
         
}


Drawer::
Drawer(uint32_t*  pointer, int  w, int  h, int  pitch):
PixelMap32(pointer,w,h,pitch)
{
}


Drawer::
Drawer(PixelMap32&&  rhs)
{
  *static_cast<PixelMap32*>(this) = std::move(rhs);
}




template<typename  T>
bool
check(T&  dst_xy, const T  dst_wh, T&  src_xy, T&  src_wh)
{
    if(dst_xy < 0)
    {
        if((dst_xy+src_wh) <= 0)
        {
          return false;
        }


      src_xy -= dst_xy;
      src_wh += dst_xy;

      dst_xy = 0;
    }

  else
    if(dst_xy >= dst_wh)
    {
      return false;
    }

  else
    if((dst_xy+src_wh) > dst_wh)
    {
      src_wh = (dst_wh-dst_xy);
    }


  return true;
}




void
Drawer::
draw_vstripe(uint32_t  color, int  x, int  y, int  length)
{
  auto  it = get_iterator(x,y);

    while(length--)
    {
      *it.get() = color;
    }
}


void
Drawer::
draw_stripe(uint32_t  color, int  x, int  y, int  length)
{
  auto  p = &get_pixel(x,y);

    while(length--)
    {
      *p++ = color;
    }
}


void
Drawer::
draw_rect(uint32_t  color, int  x, int  y, int  w, int  h)
{
  auto     top = &get_pixel(x,y    );
  auto  bottom = &get_pixel(x,y+h-1);

    rep(xx,w)
    {
         *top++ = color;
      *bottom++ = color;
    }


  auto   left = get_iterator(x    ,y);
  auto  right = get_iterator(x+w-1,y);

    rep(yy,h)
    {
       *left.get() = color;
      *right.get() = color;
    }
}


void
Drawer::
fill_rect(uint32_t  color, int  x, int  y, int  w, int  h)
{
  auto  it = get_iterator(x,y);

    rep(yy,h)
    {
      auto  dst = it.get();

        rep(xx,w)
        {
          *dst++ = color;
        }
    }
}




void
Drawer::
subtract(uint8_t  r, uint8_t  g, uint8_t  b, int  x, int  y, int  w, int  h)
{
/*
    rep(yy,h){
    rep(xx,w){
      auto&  dst = *get(x+xx,y+yy).ptr32;

      uint8_t  dst_r = clamp(to_r(dst)-r,0,255);
      uint8_t  dst_g = clamp(to_g(dst)-g,0,255);
      uint8_t  dst_b = clamp(to_b(dst)-b,0,255);
      uint8_t  dst_a = to_a(dst);

      dst = to_pixel(dst_r,dst_g,dst_b,dst_a);
    }}
*/
}




void
Drawer::
save(const char*  path) const
{
  auto  f = fopen(path,"wb");

    if(f)
    {
      auto  png = png_create_write_struct(PNG_LIBPNG_VER_STRING,nullptr,nullptr,nullptr);
      auto  inf = png_create_info_struct(png);

      png_init_io(png,f);

      png_set_IHDR(png,inf,width_,height_,8,PNG_COLOR_TYPE_RGB_ALPHA,
                                            PNG_INTERLACE_NONE,
                                            PNG_COMPRESSION_TYPE_DEFAULT,
                                            PNG_FILTER_TYPE_DEFAULT);

      std::vector<const uint8_t*>  row_pointers(height_);

      auto  it = get_const_iterator(0,0);

        rep(y,height_)
        {
          row_pointers[y] = reinterpret_cast<const uint8_t*>(it.get());
        }

      
      png_set_rows(png,inf,const_cast<uint8_t**>(row_pointers.data()));

      const auto  le_transform = PNG_TRANSFORM_IDENTITY;
      const auto  be_transform = (PNG_TRANSFORM_BGR|PNG_TRANSFORM_SWAP_ALPHA);

      png_write_png(png,inf,((SDL_BYTEORDER == SDL_LIL_ENDIAN)? le_transform:be_transform),nullptr);

      png_destroy_write_struct(&png,&inf);

      fclose(f);
    }
}




