#include"flexmenu.hpp"
#include"cursor.hpp"




FlexMenu::
FlexMenu(int  page_size, int  page_number, int  item_width, int  item_height, Callback  callback, void*  data):
page_size_(page_size),
page_number_(page_number),
page_index_(0),
position_(0),
show_cursor_icon_flag_(0),
show_cursor_frame_flag_(0),
item_width_(item_width),
item_height_(item_height),
callback_(callback),
data_(data)
{
  reset();
}


FlexMenu::
FlexMenu(std::initializer_list<const char16_t*>  texts):
page_size_(texts.size()),
page_number_(1),
page_index_(0),
position_(0),
show_cursor_icon_flag_(0),
show_cursor_frame_flag_(0),
item_width_(0),
item_height_(font_size+8),
callback_(draw_text),
data_(this),
texts_(texts)
{
    for(auto  text: texts)
    {
      item_width_ = std::max(item_width_,font_size*(int)u16strlen(text));
    }


  item_width_ += 8;

  reset();
}




void
FlexMenu::
reset()
{
  width  = 20+item_width_;
  height = (item_height_*page_size_);

  area_height_ = height;

    if(page_number_ > 1)
    {
      height += font_size;
    }
}


void
FlexMenu::
set_cursor(bool  icon, bool  frame)
{
  show_cursor_icon_flag_  = icon;
  show_cursor_frame_flag_ = frame;
}


void
FlexMenu::
reset_item_index()
{
  page_index_ = 0;
  position_   = 0;
}


int
FlexMenu::
get_item_index() const
{
  return((page_size_*page_index_)+position_);
}


void*
FlexMenu::
get_data() const
{
  return data_;
}


void
FlexMenu::
set_data(void*  data)
{
  data_ = data;
}


FlexMenu::Action
FlexMenu::
process(const Controller&  ctrl)
{
  auto  act = Action::none;

       if(ctrl.test_pressing(Key::up_flag   )){if(position_                     ){  position_ -= 1;}  act = Action::up;}
  else if(ctrl.test_pressing(Key::down_flag )){if(position_   < (page_size_  -1)){  position_ += 1;}  act = Action::down;}
  else if(ctrl.test_pressing(Key::left_flag )){if(page_index_                   ){page_index_ -= 1;}  act = Action::left;}
  else if(ctrl.test_pressing(Key::right_flag)){if(page_index_ < (page_number_-1)){page_index_ += 1;}  act = Action::right;}
  else if(ctrl.test_pressing(Key::p_flag))
    {
      act = Action::decide;
    }

  else if(ctrl.test_pressing(Key::n_flag))
    {
      act = Action::cancel;
    }

  else if(ctrl.test_pressing(Key::shift_flag))
    {
      act = Action::change;
    }

  else if(ctrl.test_pressing(Key::start_flag))
    {
      act = Action::start;
    }


  return act;
}


void
FlexMenu::
draw_text(Drawer&  dr, void*  data, int  item_index, int  x, int  y)
{
  auto&  text = static_cast<const FlexMenu*>(data)->texts_[item_index];

  dr.draw_unicode(text,0,x+4,y+4);
}


void
FlexMenu::
draw(Drawer&  dr)
{
  Widget::draw(dr);

  int  x = point.x;
  int  y = point.y;

    if(page_number_ > 1)
    {
      dr.draw_formatted(0,x,y,"page %3d/%3d",page_index_+1,page_number_);

      y += font_size;
    }


  int  yy = y;

  int  i = (page_size_*page_index_);

    rep(n,page_size_)
    {
      callback_(dr,data_,i++,x+20,yy);

      yy += item_height_;
    }


    if(show_cursor_frame_flag_)
    {
      dr.draw_rect(                              0,x+20  ,y+(item_height_*position_)  ,item_width_  ,item_height_  );
      dr.draw_rect(to_pixel32(to_mono_pixel16(12)),x+20+1,y+(item_height_*position_)+1,item_width_-2,item_height_-2);
    }


    if(show_cursor_icon_flag_)
    {
      draw_cursor(dr,x,y+(item_height_*position_)-4);
    }
}




