#include"lettermenu.hpp"
#include"font.hpp"




namespace{


enum
{
  x_number = 12,
  y_number = 11,

  item_size = 20,
};


const char16_t
hiragana_letters[y_number][x_number] =
{
  {u'あ',u'い',u'う',u'え',u'お',u'　',u'は',u'ひ',u'ふ',u'へ',u'ほ'},
  {u'か',u'き',u'く',u'け',u'こ',u'　',u'ま',u'み',u'む',u'め',u'も'},
  {u'さ',u'し',u'す',u'せ',u'そ',u'　',u'や',u'　',u'ゆ',u'　',u'よ'},
  {u'た',u'ち',u'つ',u'て',u'と',u'　',u'ら',u'り',u'る',u'れ',u'ろ'},
  {u'な',u'に',u'ぬ',u'ね',u'の',u'　',u'わ',u'を',u'ん',u'っ',u'ー'},
  {u'　',u'　',u'　',u'　',u'　',u'　',u'　',u'　',u'　',u'　',u'　'},
  {u'が',u'ぎ',u'ぐ',u'げ',u'ご',u'　',u'ば',u'び',u'ぶ',u'べ',u'ぼ'}, 
  {u'ざ',u'じ',u'ず',u'ぜ',u'ぞ',u'　',u'ゃ',u'　',u'ゅ',u'　',u'ょ'},
  {u'だ',u'ぢ',u'づ',u'で',u'ど',u'　',u'ぁ',u'ぃ',u'ぅ',u'ぇ',u'ぉ'},
  {u'ぱ',u'ぴ',u'ぷ',u'ぺ',u'ぽ',u'　',u'っ',u'ー',u'　',u'　',u'　'},

};


const char16_t
katakana_letters[y_number][x_number] =
{
  {u'ア',u'イ',u'ウ',u'エ',u'オ',u'　',u'ハ',u'ヒ',u'フ',u'ヘ',u'ホ'},
  {u'カ',u'キ',u'ク',u'ケ',u'コ',u'　',u'マ',u'ミ',u'ム',u'メ',u'モ'},
  {u'サ',u'シ',u'ス',u'セ',u'ソ',u'　',u'ヤ',u'　',u'ユ',u'　',u'ヨ'},
  {u'タ',u'チ',u'ツ',u'テ',u'ト',u'　',u'ラ',u'リ',u'ル',u'レ',u'ロ'},
  {u'ナ',u'ニ',u'ヌ',u'ネ',u'ノ',u'　',u'ワ',u'ヲ',u'ン',u'ッ',u'ー'},
  {u'　',u'　',u'　',u'　',u'　',u'　',u'　',u'　',u'　',u'　',u'　'},
  {u'ガ',u'ギ',u'グ',u'ゲ',u'ゴ',u'　',u'パ',u'ピ',u'プ',u'ペ',u'ポ'},
  {u'ダ',u'ヂ',u'ヅ',u'デ',u'ド',u'　',u'ァ',u'ィ',u'ゥ',u'ェ',u'ォ'},
  {u'ザ',u'ジ',u'ズ',u'ゼ',u'ゾ',u'　',u'ャ',u'　',u'ュ',u'　',u'ョ'},
  {u'バ',u'ビ',u'ブ',u'ベ',u'ボ',u'　',u'ッ',u'ー',u'　',u'　',u'　'}
};


const char16_t
alnum_letters[y_number][x_number] =
{
  {u'A',u'B',u'C',u'D',u'E',u' ',u'F',u'G',u'H',u'I',u'J'},
  {u'K',u'L',u'M',u'N',u'O',u' ',u'P',u'Q',u'R',u'S',u'T'},
  {u'U',u'V',u'W',u'X',u'Y',u' ',u'Z',u' ',u' ',u' ',u' '},
  {u' ',u' ',u' ',u' ',u' ',u' ',u' ',u' ',u' ',u' ',u' '},
  {u'a',u'b',u'c',u'd',u'e',u' ',u'f',u'g',u'h',u'i',u'j'},
  {u'k',u'l',u'm',u'n',u'o',u' ',u'p',u'q',u'r',u's',u't'},
  {u'u',u'v',u'w',u'x',u'y',u' ',u'z',u' ',u' ',u' ',u' '},
  {u' ',u' ',u' ',u' ',u' ',u' ',u' ',u' ',u' ',u' ',u' '},
  {u'0',u'1',u'2',u'2',u'4',u' ',u'5',u'6',u'7',u'8',u'9'},
  {u' ',u' ',u' ',u' ',u' ',u' ',u' ',u' ',u' ',u' ',u' '},

};


}


LetterMenu::
LetterMenu():
Widget((item_size*x_number),font_size+(item_size*y_number)),
table_(*alnum_letters),
x_position_(0),
y_position_(0),
charset_(Charset::alnum)
{
}




void
LetterMenu::
change_charset(Charset  set)
{
  charset_ = set;

    switch(charset_)
    {
      case(Charset::alnum   ): table_ = *alnum_letters;break;
      case(Charset::hiragana): table_ = *hiragana_letters;break;
      case(Charset::katakana): table_ = *katakana_letters;break;
      default:;
    }
}


void
LetterMenu::
reset_buffer(const std::u16string&  s)
{
  buffer_ = s;

  x_position_ = 0;
  y_position_ = 0;
}


const std::u16string&
LetterMenu::
get_buffer() const
{
  return buffer_;
}


char16_t
LetterMenu::
get_char() const
{
  return get_char(x_position_,y_position_);
}


char16_t
LetterMenu::
get_char(int  x, int  y) const
{
  return table_[(x_number*y)+x];
}


void
LetterMenu::
up()
{
    if(y_position_)
    {
      --y_position_;
    }
}


void
LetterMenu::
down()
{
    if(y_position_ < (y_number-1))
    {
      ++y_position_;
    }
}


void
LetterMenu::
left()
{
    if(x_position_)
    {
      --x_position_;
    }
}


void
LetterMenu::
right()
{
    if(x_position_ < (x_number-1))
    {
      ++x_position_;
    }
}


LetterMenu::Action
LetterMenu::
process(const Controller&  ctrl)
{
  auto  act = Action::continue_;

       if(ctrl.test_pressing(Key::up_flag   )){up(   );}
  else if(ctrl.test_pressing(Key::down_flag )){down( );}
       if(ctrl.test_pressing(Key::left_flag )){left( );}
  else if(ctrl.test_pressing(Key::right_flag)){right();}
  else if(ctrl.test_pressing(Key::shift_flag))
    {
      auto  n = (int)charset_;

      ++n;

        if(static_cast<Charset>(n) == Charset::END)
        {
          n = 0;
        }


      change_charset(static_cast<Charset>(n));
    }

  else if(ctrl.test_pressing(Key::p_flag))
    {
        if(buffer_.size() < 15)
        {
          buffer_.push_back(get_char());
        }
    }

  else if(ctrl.test_pressing(Key::n_flag) && buffer_.size())
    {
      buffer_.pop_back();
    }

  else if(ctrl.test_pressing(Key::start_flag))
    {
      act = Action::finish;
    }


  return act;
}


void
LetterMenu::
draw(Drawer&  dr)
{
  Widget::draw(dr);

  int  x = point.x;
  int  y = point.y;

  dr.draw_unicode(buffer_.data(),0,x,y);

  dr.fill_rect(to_pixel32(black_color),x+font_size*buffer_.size(),y,font_size,font_size);

  y += font_size;


    rep(yy,y_number){
    rep(xx,x_number){
      dr.draw_unicode(get_char(xx,yy),to_pixel32(black_color),x+item_size*xx,
                                                              y+item_size*yy);
    }}


  dr.draw_rect(black_color,x+item_size*x_position_,
                           y+item_size*y_position_,item_size,item_size);
}




