#include"textboard.hpp"
#include"font.hpp"
#include"rep.hpp"




TextBoard::
TextBoard()
{
}


TextBoard::
TextBoard(int  column_number, int  line_number)
{
  reset(column_number,line_number);
}




void
TextBoard::
reset(int  column_number, int  line_number)
{
  cursor_.x = 0;
  cursor_.y = 0;

  column_number_ = column_number;
  line_number_   =   line_number;

  width  = (font_w*column_number+8);
  height = (font_h*  line_number+8);

  buffer_.resize(column_number*line_number);

  std::fill(buffer_.begin(),buffer_.end(),0);

  lines_.resize(line_number);

    rep(y,line_number)
    {
      lines_[y] = &buffer_[column_number*y];
    }
}


void
TextBoard::
newline()
{
    if(cursor_.y < (column_number_-1))
    {
      cursor_.x  = 0;
      cursor_.y += 1;
    }
}


void
TextBoard::
backspace()
{
    if(cursor_.x)
    {
      lines_[cursor_.y][cursor_.x--] = 0;
    }
}


void
TextBoard::
put(char16_t  c)
{
    if(c == '\n')
    {
      newline();
    }

  else
    if(cursor_.x < (column_number_-1))
    {
      lines_[cursor_.y][cursor_.x++] = c;
    }

  else
    {
      newline();
    }
}


void
TextBoard::
copy_string(const char16_t*  src, int  line_index)
{
  auto  dst = lines_[line_index];

    while(*src)
    {
      *dst++ = *src++;
    }
}


int  TextBoard::get_cursor_x() const{return cursor_.x;}
int  TextBoard::get_cursor_y() const{return cursor_.y;}

int  TextBoard::get_column_number() const{return column_number_;}
int  TextBoard::get_line_number() const{return line_number_;}


void
TextBoard::
draw(Drawer&  dr)
{
  Widget::draw(dr);

  int  x = point.x+4;
  int  y = point.y+4;

    for(auto  line: lines_)
    {
        rep(i,column_number_)
        {
          auto  c = *line++;

            if(!c)
            {
              break;
            }


          dr.draw_unicode(c,black_color,x+(font_w*i),y);
        }


      y += font_h;
    }
}




