#include"world.hpp"




World::
World(video::Device*  video_device, audio::Device*  audio_device):
video_device_(video_device),
audio_device_(audio_device),
player_(audio_device)
{
  cube::reset(16);

  Block::set_palette(0);


  flags = delay_flag;

  key_timer = 0;
  display_timer = 0;

  initialize_manage();
  initialize_naming();
  initialize_edit();
  initialize_view();


  mode = Mode::manage;

  render_function_ = &World::render_manage;
  input_function_  = &World::input_manage;
}


World::
~World()
{
}




void  World::  set_flag(uint  flag)      {flags |=  flag;}
void  World::unset_flag(uint  flag)      {flags &= ~flag;}
bool  World:: test_flag(uint  flag) const{return(flags&flag);}




void
World::
sound(SE  se) const
{
    if(audio_device_)
    {
        switch(se)
        {
          case(SE::move_cursor):
            audio_device_->new_player(audio::WaveForm::square,4,60);
            break;
          case(SE::decide):
            audio_device_->new_player(audio::WaveForm::sawtooth,4,60);
            break;
          case(SE::cancel):
            audio_device_->new_noise(120);
            break;
        }
    }
}




bool
World::
test_key_timer(int  v)
{
    if(++key_timer >= v)
    {
      key_timer = 0;

      return true;
    }


  return false;
}


/*
MenuHandler&
World::
get_menu_handler()
{
  return *menu_handler_stack_.back();
}
*/

void
World::
push_menu_handler(/*MenuHandler&  handler*/)
{
/*
    if(menu_handler_stack_.size())
    {
      menu_handler_stack_.back()->change_status(MenuHandler::Status::warm);
    }


  menu_handler_stack_.emplace_back(&handler);

  handler.change_status(MenuHandler::Status::hot);
*/
}


void
World::
pop_menu_handler()
{
/*
    if(menu_handler_stack_.size())
    {
      menu_handler_stack_.back()->change_status(MenuHandler::Status::tepid);

      menu_handler_stack_.pop_back();

        if(menu_handler_stack_.size())
        {
          menu_handler_stack_.back()->change_status(MenuHandler::Status::hot);
        }
    }
*/
}




void
World::
change_mode(Mode  _mode)
{
    if(mode != _mode)
    {
      mode = _mode;

         mode_state = 0;
          key_timer = 0;
      display_timer = 0;

      unset_flag(paused_flag);

      set_flag(redraw_flag);
      set_flag(keyrest_flag);

        switch(mode)
        {
          case(Mode::edit):
            render_function_ = &World::render_edit;
            input_function_  = &World::input_edit;
            break;
          case(Mode::view):
            render_function_ = &World::render_view;
            input_function_  = &World::input_view;
            break;
          case(Mode::manage):
            render_function_ = &World::render_manage;
            input_function_  = &World::input_manage;
            break;
          case(Mode::naming):
            render_function_ = &World::render_naming;
            input_function_  = &World::input_naming;
            break;
        }
    }
}


void
World::
render()
{
    if(display_timer)
    {
      --display_timer;

        if(!display_timer)
        {
          set_flag(redraw_flag);
        }
    }


    if(video_device_ && test_flag(redraw_flag))
    {
      unset_flag(redraw_flag);

      video_device_->lock();

      (this->*render_function_)();

      video_device_->unlock();
    }
}




void
World::
save_screenshot(Drawer&  dr)
{
  static int  n;

  char  buf[256];

  snprintf(buf,sizeof(buf),"__SCREENSHOT%02d.bmp",n++);

  dr.save(buf);
}




void
World::
process(Controller&  ctrl)
{
    if(!ctrl.is_pressing_any())
    {
      unset_flag(keyrest_flag);
    }


    if(!test_flag(keyrest_flag))
    {
      (this->*input_function_)(ctrl);
    }


  render();


  ctrl.clear();

    if(test_flag(delay_flag))
    {
      SDL_Delay(20);
    }
}





