#ifndef WORLD_HPP
#define WORLD_HPP


#include"impr_space.hpp"
#include"widget.d/textmenu.hpp"
#include"widget.d/lettermenu.hpp"
#include"widget.d/messageboard.hpp"
#include"player.hpp"
#include"device.hpp"
#include"audio.hpp"




class
World
{
public:
  enum class Mode{
    manage,
    naming,
    edit,
    view,
  } mode;


  enum{
      redraw_flag=0x001,
     keyrest_flag=0x002,
      paused_flag=0x004,
       delay_flag=0x008,
  };


private:
  using uint = unsigned int;

  enum class SE
  {
    move_cursor,
    decide,
    cancel,
  };


  video::Device*  video_device_;
  audio::Device*  audio_device_;

  Player  player_;

  LetterMenu  letter_menu;
  TextMenu    manage_menu;
  TextMenu      edit_menu;
  TextMenu      view_menu;

  MessageBoard   name_it_board_;
  MessageBoard      name_board_;

  std::u16string  name_buffer;


  uint32_t  flags;

  int     mode_state;
  int  display_timer;
  int      key_timer;


  void  sound(SE  se) const;

  void  reset_edit();
  void  reset_view();
  void  reset_naming();


  void  initialize_manage();
  void  initialize_naming();
  void  initialize_edit();
  void  initialize_view();


  bool  test_key_timer(int  v);

//  MenuHandler&  get_menu_handler();

  void  push_menu_handler(/*MenuHandler&  handler*/);
  void   pop_menu_handler(                     );


  void  (World::*render_function_)();
  void  (World::*input_function_ )(Controller&  ctrl);

public:
   World(video::Device*  video_device, audio::Device*  audio_device);
  ~World(                                                          );


  void  change_mode(Mode  _mode);


  void  render_view();
  void  render_manage();
  void  render_naming();
  void  render_edit();

  void    set_flag(uint  flag)      ;
  void  unset_flag(uint  flag)      ;
  bool   test_flag(uint  flag) const;

  void  input_manage(Controller&  ctrl);
  void  input_edit( Controller&  ctrl);
  void  input_naming( Controller&  ctrl);
  void  input_view( Controller&  ctrl);

  static void  save_screenshot(Drawer&  dr);

  void  render();

  void  process(Controller&  ctrl);

  void  load()      ;
  void  save() const;

};




#endif




